@@ -8,6 +8,7 @@ import inspect
 import io
 import logging
 import re
+import sys
 import warnings
 from unittest.mock import MagicMock, patch
 from urllib.parse import unquote
@@ -38,6 +39,7 @@ logging.basicConfig(
 )
 logger = logging.getLogger(__name__)
 fake = faker.Faker()
+PY38 = sys.version_info >= (3, 8)
 
 
 def test_is_api_model():
@@ -50,22 +52,16 @@ def test_is_api_model():
 @pytest.mark.asyncio
 async def test_dn_property_encoder_checks_module_name(random_name):
     dn = random_name()
-    str_prop = base_http.DnPropertyEncoder(
-        random_name(), dn, random_name(), random_name()
-    ).decode()
+    str_prop = base_http.DnPropertyEncoder(random_name(), dn, random_name(), random_name()).decode()
     assert str(str_prop) == dn
     with pytest.raises(UnknownModuleType):
         await str_prop.obj
 
 
-@pytest.mark.parametrize(
-    "input_data,expected", [(None, None), ("", None)], ids=["None", "''"]
-)
+@pytest.mark.parametrize("input_data,expected", [(None, None), ("", None)], ids=["None", "''"])
 def test_dn_property_encoder_handles_empty_dn(random_name, input_data, expected):
     dn = input_data
-    str_prop = base_http.DnPropertyEncoder(
-        random_name(), dn, random_name(), random_name()
-    ).decode()
+    str_prop = base_http.DnPropertyEncoder(random_name(), dn, random_name(), random_name()).decode()
     assert str_prop is None
 
 
@@ -86,10 +82,20 @@ async def test_deepcopy_object(user_created_via_http, udm_kwargs):
     obj2 = copy.deepcopy(obj)
     assert isinstance(obj, base_http.UdmObject)
     assert isinstance(obj2, base_http.UdmObject)
-    for k in ("dn", "options", "policies", "position", "superordinate"):
-        assert getattr(obj, k) == getattr(obj2, k)
+    assert isinstance(obj.dn, str)
+    assert isinstance(obj2.dn, str)
+    assert isinstance(obj.options, dict)
+    assert isinstance(obj2.options, dict)
+    assert isinstance(obj.policies, dict)
+    assert isinstance(obj2.policies, dict)
+    assert isinstance(obj.position, str)
+    assert isinstance(obj2.position, str)
     assert isinstance(obj.props, base_http.UdmObjectProperties)
     assert isinstance(obj2.props, base_http.UdmObjectProperties)
+    assert isinstance(obj.superordinate, (str, type(None)))
+    assert isinstance(obj2.superordinate, (str, type(None)))
+    for k in ("dn", "options", "policies", "position", "superordinate"):
+        assert getattr(obj, k) == getattr(obj2, k)
     for k, v in obj.props._to_dict().items():
         assert getattr(obj.props, k) == getattr(obj2.props, k)
 
@@ -177,11 +183,14 @@ async def test_openapi_method(udm_kwargs):
 async def test_get(random_name):
     val = random_name()
 
-    async def _func():
+    async def _func():  # pragma: no-cover-py-gte-38
         return val
 
     with patch.object(base_http.UdmModule, "_load_udm_object") as load_mock:
-        load_mock.return_value = _func()
+        if PY38:  # pragma: no-cover-py-lt-38
+            load_mock.return_value = val
+        else:  # pragma: no-cover-py-gte-38
+            load_mock.return_value = _func()
         foo = await base_http.UdmModule("users/user", MagicMock()).get("foo")
         assert foo == val
 
@@ -196,7 +205,7 @@ async def test_get_none():
 async def test_dn_property_encoder(random_name):
     val = random_name()
 
-    async def _func():
+    async def _func():  # pragma: no-cover-py-gte-38
         return val
 
     property_name = random_name()
@@ -204,10 +213,11 @@ async def test_dn_property_encoder(random_name):
     session = MagicMock()
     udm_module = "users/user"
     with patch.object(base_http.UdmModule, "_load_udm_object") as load_mock:
-        load_mock.return_value = _func()
-        str_prop = base_http.DnPropertyEncoder(
-            property_name, dn, session, udm_module
-        ).decode()
+        if PY38:  # pragma: no-cover-py-lt-38
+            load_mock.return_value = val
+        else:  # pragma: no-cover-py-gte-38
+            load_mock.return_value = _func()
+        str_prop = base_http.DnPropertyEncoder(property_name, dn, session, udm_module).decode()
         assert hasattr(str_prop, "obj")
         obj = await str_prop.obj
         load_mock.assert_called_with(dn)
@@ -267,9 +277,7 @@ async def test_get_no_object(user_created_via_http, udm_kwargs):
 
 
 @pytest.mark.asyncio
-async def test_dn_property_encoder_user_group_obj(
-    user_created_via_http, udm_kwargs, base_dn
-):
+async def test_dn_property_encoder_user_group_obj(user_created_via_http, udm_kwargs, base_dn):
     dn, url, user = user_created_via_http()
 
     async with UDM(**udm_kwargs) as udm:
@@ -443,7 +451,7 @@ async def test_add_attribute_of_previously_deactivated_option(
     async with UDM(**udm_kwargs) as udm:
         mod = udm.get("shares/share")
         obj = await mod.new()
-        obj.options = ["samba"]
+        obj.options = {"samba": True}
         obj.props.name = fake.first_name()
         obj.props.host = "file.example.com"
         obj.props.path = f"/home/share{fake.first_name()}"
@@ -451,7 +459,7 @@ async def test_add_attribute_of_previously_deactivated_option(
 
         obj_new = await mod.get(obj.dn)
         assert obj_new.props.name == obj.props.name
-        if "nfs" in obj.options:
+        if obj.options.get("nfs") is True:
             # handle http://forge.univention.org/bugzilla/show_bug.cgi?id=50974
             print("NFS enabled by default :/")
             import requests
@@ -475,16 +483,16 @@ async def test_add_attribute_of_previously_deactivated_option(
                 print(resp.text)
             assert resp.status_code in (201, 204)
             obj_new = await mod.get(obj.dn)
-        assert "samba" in obj_new.options
-        assert "nfs" not in obj_new.options
+        assert obj_new.options.get("samba") is True
+        assert obj_new.options.get("nfs") is False
         assert not hasattr(obj_new.props, "root_squash")
 
-        obj_new.options.append("nfs")
+        obj_new.options["nfs"] = True
         obj_new.props.root_squash = True
         await obj_new.save()
 
         obj_new2 = await mod.get(obj_new.dn)
-        assert "nfs" in obj_new2.options
+        assert obj_new2.options.get("nfs") is True
         assert hasattr(obj_new2.props, "root_squash")
         assert obj.props.root_squash is True
 
@@ -552,9 +560,7 @@ async def test_move_and_modify_user(new_cn, user_created_via_http, udm_kwargs):
 
 
 @pytest.mark.asyncio
-async def test_move_multiple_objects(
-    base_dn, new_cn, user_created_via_http, udm_kwargs
-):
+async def test_move_multiple_objects(base_dn, new_cn, user_created_via_http, udm_kwargs):
     top_cn_dn, top_cn_obj_url, top_cn_data = new_cn()
     old_cn_dn, old_cn_obj_url, old_cn_data = new_cn(position=top_cn_dn)
     cn_name = old_cn_data["properties"]["name"]
@@ -796,8 +802,7 @@ def test_session_warn_min_client_tasks():
             udm = UDM("A", "B", "https://foo.bar/baz", max_client_tasks=i)
             assert udm.session.max_client_tasks >= 4
             assert (
-                udm.session.openapi_client_config.connection_pool_maxsize
-                >= udm.session.max_client_tasks
+                udm.session.openapi_client_config.connection_pool_maxsize >= udm.session.max_client_tasks
             )
             if i < 4:
                 assert len(w) == 1
@@ -816,8 +821,7 @@ def test_session_warn_min_connection_pool_maxsize():
                 connection_pool_maxsize=i - 1,
             )
             assert (
-                udm.session.openapi_client_config.connection_pool_maxsize
-                >= udm.session.max_client_tasks
+                udm.session.openapi_client_config.connection_pool_maxsize >= udm.session.max_client_tasks
             )
             assert len(w) == 1
             assert issubclass(w[-1].category, base_http.BadSettingsWarning)
@@ -920,9 +924,7 @@ async def test_bad_module_name(udm_kwargs):
 
 
 @pytest.mark.asyncio
-async def test_get_users_self_redirects_to_users_user(
-    udm_kwargs, test_server_configuration
-):
+async def test_get_users_self_redirects_to_users_user(udm_kwargs, test_server_configuration):
     administrator_dn = test_server_configuration.user_dn
 
     async with UDM(**udm_kwargs) as udm:
